// Export a multi-page PDF alphabet book. 

import processing.pdf.*;
boolean bExportingPDF;  
// See https://processing.org/reference/libraries/pdf/index.html

int nPages; 
PImage imageArray[];
JSONArray jsonPages;
int outputPageCount = 0; 

PFont highQualityFont32;
PFont highQualityFont100;
float inch = 72; 

//=======================================================
void setup() {

  // The book format is 6" x 9". 
  // Each inch is 72 pixels (or points). 
  // 6x72=432, 9*72=648 

  // USE THESE COMMANDS WHEN YOU'RE NOT EXPORTING PDF, 
  // AND YOU JUST WANT SCREEN DISPLAY:
  size(432, 648); 
  bExportingPDF = false;
  //
  // BUT: IMPORTANT: 
  // WHEN IT IS TIME TO EXPORT THE PDF, 
  // COMMENT OUT THE ABOVE size() COMMAND, AND
  // USE THE FOLLOWING TWO COMMANDS INSTEAD:
  // size(432, 648, PDF, "myAlphabetBook.pdf");
  // bExportingPDF = true;

  //--------------------
  // For high-quality (vector) typography:
  // See https://processing.org/reference/libraries/pdf/index.html
  // "Another option for type, is to use createFont() with a TrueType font 
  // (some OpenType fonts also work). Any font that shows up in PFont.list() 
  // should work, or if not, adding a .ttf file to the data directory 
  // and calling createFont("fontname.ttf") will also work.
  highQualityFont32 = createFont("vag.ttf", 32);
  highQualityFont100 = createFont("vag.ttf", 100);

  // Load the JSON from the data folder.
  // It looks like this: 
  /*
  [
   {
   "title": "A",
   "image": "a.jpg",
   "caption": "Ant"
   },
   {
   "title": "B",
   "image": "b.jpg",
   "caption": "Butterfly"
   }
   ]
   */
  jsonPages = loadJSONArray("book_content.json");
  nPages = jsonPages.size(); 

  // Load the images from the data folder,
  // Using the filenames from the JSON array. 
  imageArray = new PImage[nPages];
  for (int i=0; i<nPages; i++) {
    JSONObject aPage = jsonPages.getJSONObject(i);
    String imageFileName = "images/" + aPage.getString("image"); 
    imageArray[i] = loadImage(imageFileName);
  }
}

//=======================================================
void draw() {
  if (bExportingPDF) {
    drawForPDFOutput();
  } else {
    drawForScreenOutput();
  }
}

//=======================================================
void drawForPDFOutput() {

  // When finished drawing, quit and save the file
  if (outputPageCount >= nPages) {
    exit();
  } else {
    drawPage(outputPageCount); 
    PGraphicsPDF pdf = (PGraphicsPDF) g;  // Get the renderer
    if (outputPageCount < (nPages-1)) {
      pdf.nextPage();  // Tell it to go to the next page
    }
  }
  
  outputPageCount++;
}


//=======================================================
void drawForScreenOutput() {
  int whichPageIndex = (int) map(mouseX, 0, width, 0, nPages);
  drawPage(whichPageIndex);
}



//=======================================================
void drawPage(int whichPageIndex) {
  background(255);
  whichPageIndex = constrain(whichPageIndex, 0, nPages-1); 
  JSONObject whichPageObject = jsonPages.getJSONObject(whichPageIndex); 

  // Fetch and render image
  PImage whichImage = imageArray[whichPageIndex];
  float imgWRaw = whichImage.width; 
  float imgHRaw = whichImage.height;
  float imgW = max(imgWRaw, width*0.85);
  float imgScale = imgW/imgWRaw;
  float imgH = imgHRaw* imgScale;
  float imgX = (width - imgW)/2.0;
  float imgY = height - imgH - inch*1.5;
  image(whichImage, imgX, imgY, imgW, imgH); 

  // Display title letter
  String titleLetter = whichPageObject.getString("title"); 
  pushMatrix();
  float rx = random(inch*1.25, width-inch*1.25); 
  float ry = random(inch*1.25, height/3 -inch*1.25); 
  translate(rx, ry);
  noStroke();
  float randomR = random(100, 255); 
  float randomG = random(100, 255); 
  float randomB = random(100, 255); 
  fill(randomR, randomG, randomB, 100);
  ellipseMode(CENTER); 
  ellipse(0, 0, inch*2, inch*2); 
  fill(255, 255, 255); 
  textFont(highQualityFont100); 
  textAlign(CENTER); 
  text(titleLetter, 0, inch*0.5);
  popMatrix(); 

  // Assemble body text
  String captionString = whichPageObject.getString("caption"); 
  String bodyText = titleLetter + " is for " + captionString;

  // Display body text
  textFont(highQualityFont32);
  fill(0, 0, 0); 
  textAlign(CENTER); 
  text(bodyText, width/2.0, height-inch*0.6);
}
